"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = require("aws-sdk");
const FeatureNotSupportedError_1 = require("./FeatureNotSupportedError");
const ResourceNotFoundError_1 = require("./ResourceNotFoundError");
function getServiceUrl(service, region) {
    return `${service}.${region}.amazonaws.com`;
}
exports.getServiceUrl = getServiceUrl;
function assumeRole(sts, profile) {
    return __awaiter(this, void 0, void 0, function* () {
        const creds = profile ? new aws_sdk_1.SharedIniFileCredentials({ profile, }) : aws_sdk_1.config.credentials;
        if (profile && (!creds.accessKeyId || !creds.secretAccessKey)) {
            const data = yield sts.assumeRole({
                RoleArn: creds.roleArn,
                RoleSessionName: "elastic-plugin"
            }).promise();
            return {
                accessKeyId: data.Credentials.AccessKeyId,
                secretAccessKey: data.Credentials.SecretAccessKey,
                sessionToken: data.Credentials.SessionToken
            };
        }
        return Promise.resolve({
            accessKeyId: creds.accessKeyId,
            secretAccessKey: creds.secretAccessKey,
            sessionToken: creds.sessionToken
        });
    });
}
exports.assumeRole = assumeRole;
function findCloudformationExport(cf, exportName) {
    const find = (NextToken) => {
        return cf.listExports({ NextToken }).promise()
            .then((results) => {
            const item = results.Exports.find((cfExport) => cfExport.Name === exportName);
            return (item) ? item.Value : (results.NextToken) ? find(results.NextToken) : undefined;
        });
    };
    return find();
}
exports.findCloudformationExport = findCloudformationExport;
function parseConfigObject(cf, stackName, configObject) {
    return __awaiter(this, void 0, void 0, function* () {
        if (isNotDefined(configObject)) {
            return configObject;
        }
        if (typeof configObject !== "object") {
            return configObject;
        }
        const keys = Object.keys(configObject);
        const newObject = Array.isArray(configObject) ? configObject.slice() : Object.assign({}, configObject);
        for (const key of keys) {
            const item = configObject[key];
            if (!isNotDefined(item)) {
                if (Array.isArray(item)) {
                    newObject[key] = yield parseConfigObject(cf, stackName, item);
                }
                else if (typeof item === "object") {
                    const internalObjectKeys = Object.keys(item);
                    if (internalObjectKeys.length === 1 && cloudFormationObjectKeys.indexOf(internalObjectKeys[0]) >= 0) {
                        newObject[key] = yield retrieveCloudFormationValue(cf, stackName, item);
                    }
                    else {
                        newObject[key] = yield parseConfigObject(cf, stackName, item);
                    }
                }
            }
        }
        return newObject;
    });
}
exports.parseConfigObject = parseConfigObject;
const cloudFormationObjectKeys = Object.keys({
    Ref: ""
});
function retrieveCloudFormationValue(cf, stackName, value) {
    if (isNotDefined(value)) {
        return Promise.resolve(value);
    }
    if (typeof value !== "object" || Object.keys(value).length !== 1) {
        return Promise.reject(new Error("Value is not a CloudFormation parsable object."));
    }
    if (!isNotDefined(value.Ref)) {
        return findPhysicalID(cf, stackName, value.Ref);
    }
    return Promise.reject(new FeatureNotSupportedError_1.default(`CloudFormation value ${Object.keys(value)[0]} not currently supported.`));
}
exports.retrieveCloudFormationValue = retrieveCloudFormationValue;
function findPhysicalID(cf, stackName, ref) {
    const find = (NextToken) => {
        return cf.listStackResources({ StackName: stackName, NextToken }).promise()
            .then((results) => {
            const item = results.StackResourceSummaries.find(resource => resource.LogicalResourceId === ref);
            return (item) ? item.PhysicalResourceId : (results.NextToken) ? find(results.NextToken) : undefined;
        });
    };
    return find()
        .then((physicalId) => {
        if (isNotDefined(physicalId)) {
            throw new ResourceNotFoundError_1.default(`Physical ID not found for ref "${ref}" in stack "${stackName}".`);
        }
        return physicalId;
    });
}
exports.findPhysicalID = findPhysicalID;
function isNotDefined(item) {
    return item == null;
}
