"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Network_1 = require("./Network");
function discoverRepoArn(sts, repo) {
    return __awaiter(this, void 0, void 0, function* () {
        const repoCopy = Object.assign({}, repo);
        if (repo.settings.role_name) {
            const accountId = yield sts.getCallerIdentity({}).promise().then((result) => result.Account);
            const partition = getPartition(repo.settings.region);
            repoCopy.settings = Object.assign({}, repoCopy.settings);
            repoCopy.settings.role_arn = `arn:${partition}:iam::${accountId}:role/${repo.settings.role_name}`;
            delete repoCopy.settings.role_name;
        }
        return repoCopy;
    });
}
exports.discoverRepoArn = discoverRepoArn;
function getPartition(region) {
    if (region.startsWith("cn-")) {
        return "aws-cn";
    }
    if (region.startsWith("us-gov")) {
        return "aws-us-gov";
    }
    return "aws";
}
function setupRepo(props) {
    const { baseUrl, repos, sts, requestOptions, credentials } = props;
    const setupPromises = (repos || []).map((repo) => {
        validateRepo(repo);
        const { name } = repo;
        const url = `${baseUrl}/_snapshot/${name}`;
        const modifyRepo = (isS3Repository(repo)) ? discoverRepoArn(sts, repo) : Promise.resolve(repo);
        return modifyRepo.then((repo) => Network_1.esPut(url, {
            type: repo.type,
            settings: repo.settings
        }, requestOptions, credentials));
    });
    return Promise.all(setupPromises);
}
exports.setupRepo = setupRepo;
function isS3Repository(repo) {
    return repo.type === "s3";
}
function validateRepo(repo) {
    if (!repo.name) {
        throw new Error("Repo does not have a name.");
    }
    if (!repo.type) {
        throw new Error("Repo does not have a type.");
    }
}
